import __main__ as main_mod
import datetime
import numpy as np
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn
import os
main_mod.datetime = datetime
main_mod.np = np
main_mod.pd = pd
main_mod.mpl = mpl
main_mod.plt = plt
main_mod.os = os
main_mod.seaborn = seaborn
